// smart_assert.h

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_HPP_INCLUDED

#include <stdlib.h>
#include <iostream>
#include <exception>
#include "smart_assert_defs.hpp"
#include "smart_assert_settings.hpp"
#include "smart_assert_print.hpp"
#include "smart_assert_context.hpp"
#include "smart_assert_loggers.hpp"
#include "smart_assert_keeper.hpp"
#include "smart_assert_util.hpp"
#include "boost/config.hpp"


#ifdef BOOST_SMART_ASSERT_MODE
// the user has defined the mode for SMART_ASSERTs:
//
// 1 - use SMART_ASSERTs
//
// 0 - ignore SMART_ASSERTs 
// (like regular 'assert's, in release mode)
    #if BOOST_SMART_ASSERT_MODE != 0
        // (custom) debug
        #undef BOOST_SMART_ASSERT_DEBUG
        #define BOOST_SMART_ASSERT_DEBUG
    #else
        // (custom) release
        #undef BOOST_SMART_ASSERT_DEBUG
    #endif

#else

// default
    #ifndef NDEBUG
        // debug
        #undef BOOST_SMART_ASSERT_DEBUG
        #define BOOST_SMART_ASSERT_DEBUG
    #else
        // release
        #undef BOOST_SMART_ASSERT_DEBUG
    #endif
#endif



#ifdef BOOST_MSVC
// *this used in initializer list - this is ok ;-)
#pragma warning( disable: 4355)
#endif


namespace boost {

namespace smart_assert {


// in case the user does not need to customize the Assert...
struct no_custom_assert_class {};

// forward declaration(s)
namespace Private {
    class keeper;
}


/*
    The default Assert class.
    You can customize it, by (eventually) deriving from it,
    and providing your own custom make_assert function.
*/
template< 
    class printer = default_printer, 
    class logger = default_logger,
    class assert_base = no_custom_assert_class >
struct Assert 
: public assert_base {

    friend class Private::keeper;

    // helpers, in order to be able to compile the code
    Assert & SMART_ASSERT_A;
    Assert & SMART_ASSERT_B;

    Assert() 
        : m_bIgnoreThisAssert( false), 
          m_nLevel( lvl_debug), /* default level */ 
          SMART_ASSERT_A( *this),
          SMART_ASSERT_B( *this)
    {}

    ~Assert()  {
        if ( m_bIgnoreThisAssert)
            return;

        // note:
        // we do the real hanlding of the assertion right here;
        // this is because only here we have ALL of the context
        // information (for instance, the user can, at any time,
        // add a context, set the debug level, or whatever else)

        // note2:
        // you might consider it very dangerous to handle the assert 
        // so late - in a destructor, since something might throw;
        // this is true, but I cannot handle it any sooner, 
        // since I don't know when the user has set all the information
        // 
        // For instance, we might have:
        // SMART_ASSERT(i != j)(i)(j)
        //      .context("func", "get_app()").debug( "i different from j");
        //
        // or simply:
        // SMART_ASSERT(i != j)(i)(j)
        //      .context("func", "get_app()");
        // 
        // or
        // SMART_ASSERT(i != j)(i)(j)
        //      .debug( "i different from j");
        //
        // or
        // SMART_ASSERT(i != j)(i)(j);
        //
        //
        // throwing an exception from Assert's destructor
        // is not as problematic as you might think.
        // In our case, we cannot be within the context of unwinding
        // as exception, since we guard against every possible exception

        try {
            m_logger.log( m_context);
        }
        catch(const std::exception & exc) {
            assert_settings().get_dumper() << "[SMART_ASSERT logical error] "
                << "your logger class is implemented wrong.\n"
                << "it should NEVER throw an exception: '" 
                << exc.what() << "'" << std::endl;
        }
        catch(...) {
            assert_settings().get_dumper() << "[SMART_ASSERT logical error] "
                << "your logger class is implemented wrong.\n"
                << "it should NEVER throw an exception" << std::endl;
        }

        // handle the failed ASSERTion
        assert_settings().get_level_handler( m_context.get_level())( m_context);
    }


    // adds something to the current context
    template< class type>
    Assert & context( const char * strMsg, const type & val) {
        try {
            m_context.add_context_val( strMsg, val);
        }
        catch ( ...) {
            set_ignore_this_assert();
            throw;
        }
        return *this;
    }

    // sets the level message
    Assert & msg( const char * strMsg) {
        try {
            m_context.set_level_msg( strMsg);
        }
        catch ( ...) {
            set_ignore_this_assert();
            throw;
        }
        return *this;
    }


    // sets the level of the assertion
    //
    // strMsg - extra information (optional)
    //
    // FIXME (possible improvement) 
    // - allow std::string as well TOTHINK
    Assert & level( int nLevel, const char * strMsg = "") {
        try {
            m_context.set_level( nLevel);
            m_context.set_level_msg( strMsg);
        }
        catch ( ...) {
            set_ignore_this_assert();
            throw;
        }
        return *this;
    }

    // helper, similar to 'level( lvl_debug, msg)'
    Assert & debug( const char * strMsg = "") {
        return level( lvl_debug, strMsg);
    }

    // helper, similar to 'level( lvl_warn, strMsg)'
    Assert & warn( const char * strMsg = "") {
        return level( lvl_warn, strMsg);
    }

    // helper, similar to 'level( lvl_err, strMsg)'
    Assert & error( const char * strMsg = "") {
        return level( lvl_error, strMsg);
    }

    // helper, similar to 'level( lvl_critical, strMsg)'
    Assert & critical( const char * strMsg = "") {
        return level( lvl_critical, strMsg);
    }

    
    // prints the current value
    template< class type>
    Assert & print_current_val( const type & val, const char * str)
    {
        try {
            m_printer.print_val( m_context, val, str);
        }
        catch (...) {
            set_ignore_this_assert();
            throw;
        }
        return *this;
    }

    // logs the expression and 
    // the values that were pre-written using the v_ macro
    Assert & print_prewrittern_values( const Private::keeper & val) {
        try {
            // first, log the expression
            m_context.set_expr( val.get_expr());

            // now, log the values written using the v_ macro
            typedef Private::keeper::values_array values_array;
            values_array::const_iterator
                first = val.get_values_array().begin(), last = val.get_values_array().end();
            while ( first != last) {
                m_printer.print_val( m_context, 
                    // the value, then its corresponding string
                    //
                    // note: the value's type has been lost,
                    // on Private::keeper::log_current_var
                    first->first, first->second.c_str());
                ++first;
            }
        }
        catch (...) {
            set_ignore_this_assert();
            throw;
        }
        return *this;
    }

    // use this to communicate that somehow, this assertion
    // should be TOTALLY IGNORED
    // (therefore, we ignore everything in the destructor)
    //
    // this should be called, in case an exception is thrown, for instance
    Assert & set_ignore_this_assert() { 
        m_bIgnoreThisAssert = true; 
        return *this;
    }

private:
    // useful only for Private::keeper class.
    //
    // we need this, in order to know how to print values
    // when the 'v_' macro is used.
    printer & get_printer() {
        return m_printer;
    }

private:
    // if set to true, we ignore this assert 
    // (therefore, we ignore everything in the destructor)
    //
    // this should be set to true, in case an exception is thrown, for instance
    bool m_bIgnoreThisAssert;

    int m_nLevel;
    // used to customize how each variable involved in the assert 
    // is printed
    printer m_printer;

    // used to customize how the information is logged, in case
    // the assertion fails
    logger m_logger;

    // the context of this assertion
    assert_context m_context;
};


#ifndef BOOST_CUSTOM_MAKE_ASSERT

inline Assert<> make_assert() {
    return Assert<>();
}

#else
// the user has defined its own boost::smart_assert::make_assert function
//
// IMPORTANT: the custom make_assert function must be within
// boost::smart_assert namespace !
#endif




#ifdef BOOST_SMART_ASSERT_DEBUG
// behave like debug
#define SMART_ASSERT(expr) \
    if ( boost::smart_assert::Private::keeper boost_private_assert_keeper = 0) ; else \
    if ( boost_private_assert_keeper.test_expr((expr),#expr) ) ; else \
        boost::smart_assert::make_assert().SMART_ASSERT_CONTEXT.print_prewrittern_values( boost_private_assert_keeper).SMART_ASSERT_A \
    /**/

#else
// behave like release (ignore all SMART_ASSERTs)
//
// FIXME - BorlandC++ might issue a warning here...
#define SMART_ASSERT(expr) \
    if ( true ) ; else \
        boost::smart_assert::make_assert().SMART_ASSERT_A \
    /**/

    // FIXME - to erase, this is just for testing...
    #ifdef BOOST_MSVC
    #pragma message( "SMART_ASSERTs are ignored (behave like release)")
    #endif

#endif

#define SMART_ASSERT_A(x) SMART_ASSERT_OP(x, B)
#define SMART_ASSERT_B(x) SMART_ASSERT_OP(x, A)

#define SMART_ASSERT_OP(x, next) \
    SMART_ASSERT_A.print_current_val((x), #x).SMART_ASSERT_ ## next \
    /**/

} // namespace smart_assert

} // namespace boost

#endif


