// smart_assert_handlers.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_HANDLERS_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_HANDLERS_HPP_INCLUDED

// default classes that do the handling

#include <stdlib.h>
#include <stdexcept>
#include "smart_assert_context.hpp"
#include "smart_assert_util.hpp"
#include "smart_assert_defs.hpp"

namespace boost { 

namespace smart_assert {

#ifdef WIN32
    #define BOOST_BREAK_INTO_DEBUGGER_DEFINED

    inline void break_into_debugger() {
        __asm { int 3 };
    }
#else
// Define your own break_into_debugger() function !!!
#endif




#ifdef BOOST_BREAK_INTO_DEBUGGER_DEFINED
    // always break on assert
    inline void debugbreak_on_assert( const assert_context & context) {
        break_into_debugger();
    }
#endif


    // ignores the assert
    inline void ignore_assert( const assert_context & context) {
    }


    // aborts on assert
    inline void abort_on_assert( const assert_context & context) {
        abort();
    }


    // prints ALL DETAILS of the assertion to console
    inline void print_details_to_console( const assert_context & context) {
        dump_all_assert_context( context, std::cout);
    }


    // prints a summary of the assertion to console
    inline void print_summary_to_console( const assert_context & context) {
        std::cout << "\n" << Private::get_typeof_level( context.get_level() ) << " in ["
            << context.get_context_val( "file")
            << ":" << context.get_context_val( "line") << "]:\n";

        if ( !context.get_level_msg().empty())
            // we have a user-friendly message
            std::cout << context.get_level_msg() << std::endl;
        else
            std::cout << "Expression: " << context.get_expr() << std::endl;
    }

    inline void throw_exc_on_assert( const assert_context & context) {
        std::ostringstream out;
        out << "\n" << Private::get_typeof_level( context.get_level() ) << " in ["
            << context.get_context_val( "file")
            << ":" << context.get_context_val( "line") << "]:\n";

        if ( !context.get_level_msg().empty())
            // we have a user-friendly message
            out << context.get_level_msg() << std::endl;
        else
            out << "Expression: " << context.get_expr() << std::endl;

        throw std::runtime_error( out.str());
    }

    inline void ask_user_on_assert_console( const assert_context & context) {
        // show info to the user
        std::cout << "\n" << Private::get_typeof_level( context.get_level() ) << " in ["
            << context.get_context_val( "file")
            << ":" << context.get_context_val( "line") << "]:\n";

        if ( !context.get_level_msg().empty())
            // we have a user-friendly message
            std::cout << context.get_level_msg() << std::endl;
        else
            std::cout << "Expression: " << context.get_expr() << std::endl;

        // read
        std::cout << "\nChoose option: D - Details, I - Ignore, R - Retry, A - Abort" << std::endl;
        bool bContinue = true;
        while ( bContinue) { 
            bContinue = false;
            char ch = std::cin.get();
            switch ( ch) {
            case 'd': case 'D':
                // details
                std::cout << std::endl;
                dump_all_assert_context( context, std::cout);
                bContinue = true;
                break;

            case 'i': case 'I':
                // ignore
                break;

            case 'r': case 'R':
                // retry
#ifdef BOOST_BREAK_INTO_DEBUGGER_DEFINED
                break_into_debugger();
#else
                // don't know how to break...
                std::cout << "Don't know how to break into debugger..." << std::endl;
                abort();
#endif
                break;

            case 'a': case 'A':
                // abort
                abort();
                break;

            default:
                bContinue = true;
                break;
            };

            if ( !isspace( ch) && bContinue)
                // ignore spaces
                std::cout << "\nChoose option: D - Details, I - Ignore, R - Retry, A - Abort" << std::endl;
        };

        // FIXME - do ignore_all;
        // note: I will assume there is ONLY ONE assertion per line
        // (reasonable assumtion), in order to be able to have
        // 'Ignore All'.
    }

    inline void ask_user_on_assert( const assert_context & context) {
#ifdef WIN32
        // FIXME - for Win32, ask with MsgBox, etc.
        // FIXME handle entering assert recursively
        ask_user_on_assert_console( context);
#else
        ask_user_on_assert_console( context);
#endif

    }


    inline void default_handle_warn( const assert_context & context) {
        print_summary_to_console( context);
    }

    inline void default_handle_debug( const assert_context & context) {
        ask_user_on_assert( context);
    }

    inline void default_handle_error( const assert_context & context) {
        throw_exc_on_assert( context);
    }

    inline void default_handle_critical( const assert_context & context) {
        dump_all_assert_context( context, std::cout );
        abort_on_assert( context);
    }


} // namespace smart_assert

} // namespace boost




#endif
