// smart_assert_util.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_UTIL_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_UTIL_HPP_INCLUDED

// Utility functions

#include <string>
#include "smart_assert_context.hpp"

namespace boost { 

namespace smart_assert {

namespace Private {

    // returns a message corresponding to the type of level
    // takes into considerance only default existing levels 
    // (does not take custom levels into account, in case the user
    //  has defined any)
    inline std::string get_typeof_level( int nLevel) {
        switch ( nLevel) {
        case lvl_warn:
            return "Warning";

        case lvl_debug:
            return "Assertion failed";

        case lvl_error:
            return "Assertion failed (Error)";

        case lvl_critical:
            return "Assertion failed (FATAL)";

        default: {
            std::ostringstream out;
            out << "Assertion failed (level=" << nLevel << ")";
            return out.str();
                 }
        };
    }

} // namespace Private


namespace Private {

    // ... used for dumping part of the assert_context
    struct log_context_val {
        log_context_val( std::ostream & out) : m_out( out) {}

        typedef std::pair< const std::string, std::string> value_type;
        void operator()( const value_type & val) {
            if ( ( val.first != "file") && 
                 ( val.first != "line") ) {
                m_out << val.first << " = '" << val.second << "';\n";
            }
        }
    private:
        std::ostream & m_out;
    };

    // ... used for dumping part of the assert_context
    struct log_val {
        log_val( std::ostream & out) : m_out( out) {}

        typedef assert_context::val_and_str value_type;
        void operator()( const value_type & val) {
            m_out << "\t" << val.m_str << " = '" << val.m_val << "';\n";
        }
    private:
        std::ostream & m_out;
    };

} // namespace Private

    // dumps all ASSERT context
    void dump_all_assert_context( const assert_context & context, std::ostream & out) {

        // log context
        out << "\n" << Private::get_typeof_level( context.get_level() ) << " in ["
            << context.get_context_val( "file")
            << ":" << context.get_context_val( "line") << "]:\n";
        if ( context.get_context_vals_collection().size() > 2) {
            // extra information
            std::for_each( 
                context.get_context_vals_collection().begin(), 
                context.get_context_vals_collection().end(),
                Private::log_context_val( out) );
        }

        // log expression
        if ( !context.get_level_msg().empty())
            out << "\n\tUser-friendly msg: '" << context.get_level_msg() << "'";
        out << "\n\tExpression = '" << context.get_expr() << "'\n";

        // log values
        out << "\n\tValues:\n";
        std::for_each( 
            context.get_vals_array().begin(), context.get_vals_array().end(),
            Private::log_val( out) );
    }


} // namespace smart_assert

} // namespace boost


#endif
